/**
* Klasse, die Onlineunterrichtsstunde steuert
*/
function Onlineunterrichtsstunde(
    objStepByStep,  // Zuständige Instanz von StepByStep
    lessonData      // Objekt mit Informationen zur Unterrichtsstunde
) {
    this.objStepByStep = objStepByStep;
    this._loadSvg(lessonData);
}

// Zuständige Instanz von StepByStep
Onlineunterrichtsstunde.prototype.objStepByStep = null;

/**
* Die verwendete SVG-Datei laden
* @return undefined
*/
Onlineunterrichtsstunde.prototype._loadSvg = function(
    lessonData  // Objekt mit Informationen zur Unterrichtsstunde
) {
    var self = this;
    var oReq = new XMLHttpRequest();
    oReq.overrideMimeType('text/xml');
    oReq.addEventListener("load", function() {
        self.importSvg(oReq.responseXML);
    });
    oReq.addEventListener("error", function() {
        self.objStepByStep.showMsg('Die Onlineunterrichtsstunde kann wegen eines unerwarteten Fehlers leider nicht geladen werden…');
    });
    oReq.addEventListener("abort", function() {
        self.objStepByStep.showMsg('Die Onlineunterrichtsstunde kann nicht geladen werden, weil du den Ladevorgang abgebrochen hast.');
    });
    oReq.open('GET', (lessonData.basedir ? (lessonData.basedir + '/') : '') + lessonData.svg);
    oReq.send();
}

/**
* Onlineunterrichtsstunde aus SVGDocument importieren, anzeigen, Navigationsleiste einblenden und Kommunikation mit LMS starten
* @return undefined
*/
Onlineunterrichtsstunde.prototype.importSvg = function(
    xml         // Document mit SVG
) {
    var svg = xml.documentElement;
    svg.classList.remove('edit');
    this._svg = document.importNode(svg, true);
    this.objStepByStep.establishContent(this._svg, this._getDirectChildrenByTagName(this._svg, 'G'));
}

/**
* Array mit direkten Kindknoten erstellen, die angegebenen Tagnamen haben
* @return Array
*/
Onlineunterrichtsstunde.prototype._getDirectChildrenByTagName = function(
    parent,     // Elternknoten
    tagname     // gewünschter Tagname in Großbuchstaben
) {
    var a = [];
    for (var node = parent.firstChild; node; node = node.nextSibling) {
        if ((node.nodeType == Node.ELEMENT_NODE) && (node.nodeName.toUpperCase() == tagname))
            a.push(node);
    }
    return a;
}

/**
* Größe der Präsentationsfläche an Fenstergröße anpassen
* @return undefined
*/
Onlineunterrichtsstunde.prototype.updateSize = function(
    width,      // Verfügbare Breite für Präsentationsfläche
    height      // Verfügbare Höhe für Präsentationsfläche (ggf. exklusive Navigationsleiste)
) { }

/**
* So scrollen, dass die aktuelle Ebene im sichtbaren Bereich liegt
* @return undefined
*/
Onlineunterrichtsstunde.prototype.scrollToActive = function(
    content,    // Aktive Gruppe
    navHeight   // Höhe der Navigationsleiste
) {
    if (!content)
        return;
    var best = undefined;
    var rect = content.getBoundingClientRect();
    if (!rect.height)
        return;
    var now = window.scrollY;
    var minY = now + rect.bottom - window.innerHeight;
    var maxY = now + rect.top - navHeight;
    if (maxY < minY) {
        if ((maxY > now) || (now > minY))
            best = maxY;
    } else if (now > maxY)
        best = maxY;
    else if (now < minY)
        best = minY + Math.max(0, Math.min((window.innerHeight - rect.bottom + rect.top) / 2, Math.min(window.innerHeight, window.innerWidth) / 10));
    if (Number.isFinite(best) && window.scroll)
        window.scroll({ top: best, behavior: 'smooth' });
}

/**
* Aktuell aktiven Inhalt korrekt anzeigen
* @return undefined
*/
Onlineunterrichtsstunde.prototype.changeActive = function(
    activeOld,      // Index des Inhalts, der bislang aktiv war (-1 möglich!)
    activeNew,      // Index des Inhalts, der neuerdings aktiv sein soll
    arrContents     // Sortiertes Array aller Inhalte, die aktiv sein können
) {
    // Richtige SVG-Elemente anzeigen
    for (var z = 0; z < arrContents.length; ++z) {
        if (arrContents[z]) {
            arrContents[z].style.opacity = (!z || (z <= activeNew)) ? 1 : 0;
            if (z == activeNew)
                arrContents[z].classList.add('active');
            else
                arrContents[z].classList.remove('active');
            if (z < activeNew)
                arrContents[z].classList.add('done');
            else
                arrContents[z].classList.remove('done');
        }
    }

    // Im SVG-Wurzelelement als Klasse speichern, welcher Schritt aktiv ist
    if (this._svg.classList) {
        if (this._svgClass)
            this._svg.classList.replace(this._svgClass, 'schritt' + (activeNew + 1));
        else
            this._svg.classList.add('schritt' + (activeNew + 1));
    } else {
        if (this._svgClass)
            this._svg.className = this._svg.className.replace(this._svgClass, 'schritt' + (activeNew + 1));
        else
            this._svg.className = this._svg.className.length ? (this._svg.className + ' schritt' + (activeNew + 1)) : ('schritt' + (activeNew + 1));
    }
    this._svgClass = 'schritt' + (activeNew + 1);
}

/**
* Modus wechseln
* @return undefined
*/
Onlineunterrichtsstunde.prototype.changeMode = function(
    newMode     // Neuer Modus: StepByStep.MODE_*
) {
    if (newMode != StepByStep.MODE_SENDER) {
        if (this._xtraStyle && this._xtraStyle.parentNode)
            this._xtraStyle.parentNode.removeChild(this._xtraStyle);
    } else if (!this._xtraStyle || !this._xtraStyle.parentNode) {
        var defs = this._svg.getElementsByTagName('defs');
        defs = defs.length ? defs[0] : ScoTools.createSvg('defs', this._svg);
        this._xtraStyle = ScoTools.createSvg('style', defs, null, 'svg .done {opacity:0.55 !important} svg .active path, svg .done path {stroke-dasharray:none !important}');
    }
}
